/*!
    \file spatial_separation.h  Definition of the generic spatial separation process of model data
 */

#ifndef _SPATIAL_SEPARATION_H
#define _SPATIAL_SEPARATION_H

#include "base_object.h"
#include "vector.h"


class AbstractModel;
class Quadtree;


#define SEC_OFFSET 1.0                       //!< the offset added to the overlapSize to ensure that there are no problems in


//! A class that represents a rectangle, this is needed for SpatialSeparation
class Rectangle
{

  public:
    Rectangle();
    virtual ~Rectangle();

    /** \brief sets the center of the rectangle to a defined vector @param center the new center */
    inline void setCenter(const Vector &center) { this->center = center;}
    /** \brief returns the center of the rectangle to a defined vector @returns center the new center */
    inline const Vector* getCenter() const { return &this->center; }

    /** \brief sets both axis of the rectangle to a defined vector @param unityLength the new center */
    inline void setAxis(float unityLength) { this->axis[0] = unityLength; this->axis[1] = unityLength; }
               /** \brief sets both axis of the rectangle to a defined vector @param v1 the length of the x axis @param v2 the length of the z axis*/
    inline void setAxis(float v1, float v2) { this->axis[0] = v1; this->axis[1] = v2; }

  private:
    Vector center;
    float axis[2];
};


//! A class for spatial separation of vertices based arrays
class SpatialSeparation : public BaseObject {

  public:
    SpatialSeparation(AbstractModel* model, float overlapSize);
    SpatialSeparation(AbstractModel* model, AbstractModel* playerModel);
    virtual ~SpatialSeparation();

    void setTreeDepth(int depth) { this->treeDepth = depth; }
    void setMinLength(int minLength) { this->minLength = minLength; }

    Quadtree* createQuadtree(AbstractModel* model, float minLength);
    Quadtree* createQuadtree(AbstractModel* model, int treeDepth);
    Quadtree* createQuadtree(AbstractModel* model);


  private:
    void separateZone();
    Rectangle* getDimension(AbstractModel* playerModel);


  private:
    AbstractModel*             model;        //!< the reference to the model that has to be handled
    Quadtree*                  quadtree;     //!< the reference to the created quadtree

    AbstractModel*             playerModel;  //!< referece to the player model, if needed for overlap calculations
    float                      overlapSize;  //!< the size of overlaping

    int                        treeDepth;    //!< depth of the quadtree
    float                      minLength;    //!< min length of a quadtree node
};

#endif /* _SPATIAL_SEPARATION_H */
